//
//  CustomAlertView.m
//  ac-service-ios-Demo
//
//  Created by fariel huang on 2017/1/5.
//  Copyright © 2017年 OK. All rights reserved.
//

#import "CustomTableAlertView.h"
#import "CommonDefine.h"
#import "UIView+Frame.h"

static const CGFloat kTableMaxHeight = 200.0f; //用于选择的tableView的最大高度
static const CGFloat kTableCellHeight = 40.0f; //用于选择的tableView的cell高度
static const CGFloat kTableWidth = 280.0f; //用于选择的tableView的宽度（固定）
static const CGFloat kTablePadding = 10.0f; //用于选择的tableView的父控件边距

static const CGFloat kTitleHeight = 45.0f; //标题高度s

static const CGFloat kButtonHeight = 40.0f; //按钮高度
static const CGFloat kButtonPadding = 10.0f; //按钮边距

static CustomTableAlertView *sharedInstance;

@interface CustomTableAlertView ()<UITableViewDelegate, UITableViewDataSource>

/** delegate */
@property (nonatomic, weak) id delegate;
/** 选项数组 */
@property (nonatomic, copy) NSArray *options;
/** 选择结果回调 */
@property (strong, nonatomic) BOOL(^callbackSelections)(NSInteger selectedIndex, NSInteger buttonIndex);
/** 下拉列表 UITableView */
@property (nonatomic, strong) UITableView *optionsTableView;
/** 选中的选项 */
@property (nonatomic, assign) NSInteger selectedIndex;
/** 背景view */
@property (nonatomic, strong) UIView *vDialog;

@end

@implementation CustomTableAlertView

+ (void)alertWithTarget:(id)target
                  title:(NSString *)title
                options:(NSArray *)options
                buttons:(NSArray<NSString *> *)buttons
               callback:(BOOL(^)(NSInteger selectedIndex,
                                 NSInteger buttonIndex))callback {
    UIWindow *window = [UIApplication sharedApplication].windows[0];
    if (sharedInstance == nil) {
        //底层蒙版
        sharedInstance = [[self alloc]
                          initWithFrame:CGRectMake(0,
                                                   -100,
                                                   window.frame.size.width,
                                                   window.frame.size.height+100)];
        sharedInstance.callbackSelections = callback;
        [window addSubview:sharedInstance];
        
        sharedInstance.alpha = 0.0f;
        sharedInstance.backgroundColor = UIColorFromRGBA(0x2C3E50,0x50);
        //dialog view
        sharedInstance.vDialog = [[UIView alloc] init];
        sharedInstance.vDialog.frame = CGRectMake(0,
                                   0,
                                   kTableWidth + kTablePadding * 2,
                                   kTitleHeight + [sharedInstance getTableHeight:options.count] + kButtonHeight + kButtonPadding * 2 + kTablePadding);
        sharedInstance.vDialog.center = CGPointMake(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2);
        sharedInstance.vDialog.layer.cornerRadius=5;
        sharedInstance.vDialog.backgroundColor = [UIColor whiteColor];
        sharedInstance.vDialog.layer.shadowOffset = CGSizeMake(0, 3);
        sharedInstance.vDialog.layer.shadowRadius = 3.0;
        sharedInstance.vDialog.layer.shadowColor = [UIColor blackColor].CGColor;
        sharedInstance.vDialog.layer.shadowOpacity = 0.8;
        [sharedInstance addSubview:sharedInstance.vDialog];
        
        //顶部中间title
        UILabel *titleLabel=[[UILabel alloc]
                             initWithFrame:CGRectMake(0,
                                                      0,
                                                      sharedInstance.vDialog.width,
                                                      kTitleHeight)];
        [titleLabel setTextAlignment:NSTextAlignmentCenter];
        [titleLabel setBackgroundColor:[UIColor clearColor]];
        [titleLabel setText:title];
        [titleLabel setTextColor:[UIColor blackColor]];
        [titleLabel setFont:[UIFont boldSystemFontOfSize:20]];
        [sharedInstance.vDialog addSubview:titleLabel];
        
        sharedInstance.optionsTableView = [[UITableView alloc]
                                           initWithFrame:CGRectMake(kTablePadding,
                                                                    titleLabel.height + titleLabel.y,
                                                                    kTableWidth,
                                                                    [sharedInstance getTableHeight:options.count])
                                           style:UITableViewStyleGrouped];
        sharedInstance.optionsTableView.delegate = sharedInstance;
        sharedInstance.optionsTableView.dataSource = sharedInstance;
        sharedInstance.optionsTableView.backgroundColor = [UIColor whiteColor];
        [sharedInstance.vDialog addSubview:sharedInstance.optionsTableView];
        sharedInstance.options = options.mutableCopy;
        //取消按钮
        CGFloat buttonWidth = (sharedInstance.vDialog.width - (buttons.count + 2) * kButtonPadding) / (buttons.count + 1);
        UIButton *cancelButton = [[UIButton alloc]
                                  initWithFrame:CGRectMake(kButtonPadding,
                                                           sharedInstance.optionsTableView.y + sharedInstance.optionsTableView.height + kButtonPadding + kTablePadding,
                                                           buttonWidth,
                                                           kButtonHeight)];
        cancelButton.backgroundColor = UIColorFromRGB(0xE74C3C);
        cancelButton.layer.cornerRadius = 3;
        [cancelButton setTitle:@"取消" forState:UIControlStateNormal];
        [cancelButton addTarget:sharedInstance
                         action:@selector(cancelButtonClicked:)
               forControlEvents:UIControlEventTouchUpInside];
        [sharedInstance.vDialog addSubview:cancelButton];
        
        //按钮集合
        [buttons enumerateObjectsUsingBlock:^(NSString * _Nonnull obj,
                                              NSUInteger idx,
                                              BOOL * _Nonnull stop) {
            UIButton *button = [[UIButton alloc]
                                initWithFrame:CGRectMake(kButtonPadding * (idx + 2) + cancelButton.width * (idx + 1),
                                                         cancelButton.y,
                                                         cancelButton.width,
                                                         kButtonHeight)];
            button.backgroundColor = UIColorFromRGB(0x25AEE5);
            button.layer.cornerRadius = 3;
            button.tag = idx;
            [button setTitle:obj forState:UIControlStateNormal];
            [button addTarget:sharedInstance
                       action:@selector(buttonClicked:)
             forControlEvents:UIControlEventTouchUpInside];
            [sharedInstance.vDialog addSubview:button];
        }];
        
        //动画显示
        [UIView animateWithDuration:0.3f animations:^{
            sharedInstance.y += 100;
            sharedInstance.alpha = 1.0f;
        } completion:nil];
        sharedInstance.delegate = target;
        sharedInstance.selectedIndex = -1;
    }
}

+ (void)dismiss {
    [UIView animateWithDuration:0.3f animations:^{
        sharedInstance.y -= 100;
        sharedInstance.alpha = 0.0f;
    } completion:^(BOOL finished) {
        [sharedInstance removeFromSuperview];
        sharedInstance = nil;
    }];
}

+ (void)updateOptions:(NSArray *)options {
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3f * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        CGFloat diffHeight = [sharedInstance getTableHeight:options.count] - [sharedInstance getTableHeight:sharedInstance.options.count];
        sharedInstance.options = options.copy;
        [sharedInstance.optionsTableView reloadData];
        [UIView animateWithDuration:0.5f animations:^{
            [sharedInstance.vDialog.subviews enumerateObjectsUsingBlock:^(__kindof UIView * _Nonnull subView,
                                                                          NSUInteger idx,
                                                                          BOOL * _Nonnull stop) {
                if ([subView isKindOfClass:[UIButton class]]) {
                    subView.y += diffHeight;
                }
            }];
            sharedInstance.optionsTableView.height += diffHeight;
            sharedInstance.vDialog.height += diffHeight;
            sharedInstance.vDialog.center = CGPointMake(WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2);
        }];
    });
}

- (CGFloat)getTableHeight:(NSInteger)count {
    return kTableCellHeight * count > kTableMaxHeight ? kTableMaxHeight : kTableCellHeight * count;
}

+ (BOOL)isDisplaying {
    return !(sharedInstance.alpha == 0 || sharedInstance == nil);
}

#pragma mark - button click events 

- (void)cancelButtonClicked:(UIButton *)sender {
    [CustomTableAlertView dismiss];
}

- (void)buttonClicked:(UIButton *)sender {
    if (_callbackSelections
        && _callbackSelections(_selectedIndex, sender.tag)) {
        [CustomTableAlertView dismiss];
    }
}
#pragma mark - UITableView Delegates & Datasource

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    //设备 cell 界面
    static NSString *cellId=@"cell";
    UITableViewCell *cell = [_optionsTableView dequeueReusableCellWithIdentifier:cellId];
    if(!cell){
        cell=[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellId];
    }
    cell.accessoryType = (indexPath.row == _selectedIndex ? UITableViewCellAccessoryCheckmark : UITableViewCellAccessoryNone);
    cell.textLabel.text = self.options[indexPath.row];
    if ([sharedInstance.delegate respondsToSelector:@selector(customTableAlertViewCell:atIndex:)]) {
        [sharedInstance.delegate customTableAlertViewCell:cell atIndex:indexPath.row];
    }
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return kTableCellHeight;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.options.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [_optionsTableView cellForRowAtIndexPath:indexPath].selected = NO;
    _selectedIndex = indexPath.row;
    [_optionsTableView reloadData];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    return FLT_MIN;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section {
    return FLT_MIN;
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self endEditing:YES];
    [CustomTableAlertView dismiss];
}

@end
