//
//  DeviceListViewController.m
//  ac-service-ios-Demo
//
//  Created by OK on 15/2/9.
//  Copyright (c) 2015年 OK. All rights reserved.
//
#import <SystemConfiguration/CaptiveNetwork.h>
#import "DeviceListViewController.h"
#import "ACWifiLinkManager.h"
#import "TimerMsgSendingViewController.h"
#import "CommonDefine.h"
#import "UserDefaultUtils.h"

//light cell
#import "DeviceListLightCelll.h"
//显示开关灯  信息
#import "ShowTimeLine.h"

//显示所需的信息
#import "DeviceMsg.h"
//缓存
#import "ACCache.h"
#import "MJRefresh.h"

#import "ACloudLib.h"

#import "MBProgressHUD+MJ.h"
#import "UIView+Frame.h"
#import "DeviceMsg.h"

#import "ACBindManager.h"
#import "ACUserDevice.h"
#import "ACBindUser.h"
#import "ACDeviceActive.h"
#import "ACDeviceManager.h"
#import "ACDeviceMsg.h"
#import "ACLocalDevice.h"
#import "ACDeviceDataManager.h"
#import "ACDevicePropertySearchOption.h"
#import "ACAccountManager.h"
#import "ACOTAManager.h"

static const NSTimeInterval kDeviceListFindLocalInterval = 5; //局域网设备刷新间隔

@interface DeviceListViewController () <DeviceListLightCelllDelegate, UIAlertViewDelegate, UIActionSheetDelegate>

@property(nonatomic,strong) NSMutableArray<DeviceMsg *> *deviceList;

@property(nonatomic,strong)NSIndexPath *selectIndexPath;

@property(nonatomic,strong)DeviceMsg *currectDevice;

@property(nonatomic,strong)NSTimer *timerFindLocal;

@end

@implementation DeviceListViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.tableView addHeaderWithTarget:self action:@selector(refreshDeviceList)];
    self.tableView.tableFooterView = [[UIView alloc]init];
    [self startTimerFindLocal];
    
    [ACAccountManager setRefreshTokenInvalidCallback:^(NSError *error) {
        [ACAccountManager logout];
        //其他需要用户重新登录的操作
        // ...
    }];
}

/**
 *  定时刷新局域网设备的数据
 */
- (void)startTimerFindLocal {
    _timerFindLocal = [NSTimer scheduledTimerWithTimeInterval:kDeviceListFindLocalInterval + 1
                                                       target:self
                                                     selector:@selector(refreshLocalDeviceList)
                                                     userInfo:nil
                                                      repeats:YES];
    [[NSRunLoop currentRunLoop] addTimer:_timerFindLocal forMode:NSRunLoopCommonModes];
}

- (void)refreshLocalDeviceList {
    [ACloudLib findDeviceTimeout:kDeviceListFindLocalInterval callback:^(NSArray *localDeviceList) {
        for (DeviceMsg *deviceMsg in self.deviceList) {
            BOOL found = NO;
            for (ACLocalDevice *localDevice in localDeviceList) {
                //注：ACLocalDevice中的deviceId即为ACUserDevice的physicalDeviceId 而非ACUserDevice的deviceId
                if ([deviceMsg.device.physicalDeviceId isEqual:localDevice.deviceId]) {
                    found = YES;
                }
            }
            deviceMsg.device.status = [self statusWithOriginStatus:deviceMsg.device.status localFound:found];
        }
        [self.tableView reloadData];
    }];
}

/**
 *  进行更新 设备的数据 （状态：下拉刷新的状态）
 */
- (void)refreshDeviceList {
    @ACWeakify(self)
    [ACBindManager listDevicesWithStatusCallback:^(NSArray<ACUserDevice *> *devices,
                                                   NSError *error) {
        @ACStrongify(self)
        NSLog(@"%@", devices);
        [self.deviceList removeAllObjects];
        for (ACUserDevice *device in devices) {
            
            DeviceMsg *deviceMsg = [[DeviceMsg alloc] init];
            deviceMsg.isLightOn = [device.properties getBool:@"switch"];
            deviceMsg.device = device;
            [self.deviceList addObject:deviceMsg];
            
            [ACDeviceDataManager subscribePropDataWithSubDomain:device.subDomain
                                                       deviceId:device.deviceId
                                                       callback:^(NSError *error) {
                                                           if (error) {
                                                               NSLog(@"订阅设备属性失败!");
                                                           }
                                                       }];
            [ACDeviceDataManager subscribeFaultsWithSubDomain:device.subDomain
                                                     deviceId:device.deviceId
                                                     callback:^(NSError *error) {
                                                         if (error) {
                                                             NSLog(@"订阅设备故障失败!");
                                                         }
                                                     }];
        }
        [self.tableView headerEndRefreshing];
        [self.tableView reloadData];
    }];
    //设置设备属性实时数据回调
    [ACDeviceDataManager setPropertyMessageHandler:^(NSString *subDomain,
                                                     NSInteger deviceId,
                                                     ACObject *properties) {
        @ACStrongify(self)
        //更新开关灯状态
        [self updateLightState:[properties getBool:@"switch"]
                 withSubDomain:subDomain
                      deviceId:deviceId];
        NSLog(@"sub device props : %td,%@,%@", deviceId, subDomain, properties);
    }];
    [ACDeviceDataManager setFaultsMessageHandler:^(NSString *subDomain, NSInteger deviceId, ACObject *faults) {
        NSLog(@"sub device faults : %td,%@,%@", deviceId, subDomain, faults);
    }];
}

- (void)beginList {
    [self.tableView headerBeginRefreshing];
}

- (void)viewWillAppear:(BOOL)animated {
    [self beginList];
    [super viewWillAppear:animated];
    [self.tableView reloadData];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(NSIndexPath *)deleteIndexPath {
    if (_selectIndexPath == nil) {
        _selectIndexPath = [[NSIndexPath alloc]init];
    }
    return _selectIndexPath;
}

#pragma mark - Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return [self.deviceList count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    DeviceListLightCelll *cell = [DeviceListLightCelll defaultTableView:tableView
                                                              deviceMsg:self.deviceList[indexPath.row]];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    cell.delegate = self;
    return cell;
    
}

/**
 *  重新设置Wi-Fi
 */
- (void)resetWiFi:(NSString *)ssid password:(NSString *)password {
    [MBProgressHUD showMessage:@"重新设置Wi-Fi"];
    ACWifiLinkManager *wifiLinkManager = [[ACWifiLinkManager alloc] initWithLinkerName:ACLinkerNameAndroid];
    NSString *physicsID = [self.deviceList[self.selectIndexPath.row].device physicalDeviceId];
    
    NSLog(@"--%@--",ssid);
    NSLog(@"--%@--",password);
    NSLog(@"--%@--",physicsID);
    
    [wifiLinkManager sendWifiInfo:ssid password:password
                 physicalDeviceId:physicsID
                          timeout:60.0
                         callback:^(NSString *deviceId, NSString *randMsg, NSError *error) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (error) {
                [MBProgressHUD hideHUD];
                [MBProgressHUD showError:@"设置失败"];
            } else {
                [MBProgressHUD hideHUD];
                [MBProgressHUD showSuccess:@"设置成功"];
            }
        });
    }];
}

#pragma mark DeviceListLightCelll 代理
- (void)deviceListLightCelll:(UITableViewCell *)cell lightOperation:(LightOperationType)operationType {
    [self operationLight:operationType device:self.deviceList[[self.tableView indexPathForCell:cell].row]];
}

#pragma mark tableView 代理
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    self.selectIndexPath = indexPath;
    self.currectDevice = self.deviceList[indexPath.row];
    UIActionSheet *sheet = [[UIActionSheet alloc]initWithTitle:@"请选择操作"
                                                      delegate:self
                                             cancelButtonTitle:@"取消"
                                        destructiveButtonTitle:@"查看设备历史纪录"
                                             otherButtonTitles:@"修改Wi-Fi", @"自定义发送消息", @"OTA-MCU", @"OTA-WIFI", nil];
    
    [sheet showInView:self.view];
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    self.selectIndexPath = indexPath;
    UIAlertView *alertView = [[UIAlertView alloc]
                              initWithTitle:@"设备删除"
                              message:@"您确定需要删除设备吗"
                              delegate:self
                              cancelButtonTitle:@"取消"
                              otherButtonTitles:@"确定", nil];
    [alertView show];
}

- (NSString*)tableView:(UITableView *)tableView titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath {
    return @"删除";
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 64.0;
}

#pragma mark UIAlertView代理
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    
    if (buttonIndex == 1 && [alertView textFieldAtIndex:0] == nil) {
        
        ACUserDevice  *device = self.deviceList[self.selectIndexPath.row].device;
        NSInteger deviceId = device.deviceId;
        NSString *subDomain = device.subDomain;
        [ACBindManager unbindDeviceWithSubDomain:subDomain
                                        deviceId:deviceId
                                        callback:^(NSError *error) {
            if(error){
                NSLog(@"unbindDevice--error:%@",error);
            }else{
                NSLog(@"unbindDevice--OK");
            }
        }];
        
        [self.deviceList removeObjectAtIndex:[self.selectIndexPath row]];
        [self.tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:self.selectIndexPath]
                              withRowAnimation:UITableViewRowAnimationLeft];
        return;
    }
    
    NSString *SSID = [[alertView textFieldAtIndex:0] text];
    NSString *password = [[alertView textFieldAtIndex:1] text];
    if (buttonIndex == 1) {
        [self resetWiFi:SSID password:password];
    }
}

- (void)alertView:(UIAlertView *)alertView willDismissWithButtonIndex:(NSInteger)buttonIndex {
    NSLog(@"dismiss");
}

/**
 *  网络连接操作灯
 */
- (void)operationLight:(LightOperationType)type device:(DeviceMsg *)deMsg {
    
    ACDeviceMsg *msg = [[ACDeviceMsg alloc]init];
    msg.msgCode = 68;
    Byte content[] ={type,0,0,0};
    msg.payload = [NSData dataWithBytes:content length:sizeof(content)];
    [ACBindManager sendToDeviceWithOption:ACDeviceCommunicationOptionLocalFirst
                                SubDomain:deMsg.device.subDomain
                         physicalDeviceId:deMsg.device.physicalDeviceId
                                      msg:msg
                                 callback:^(ACDeviceMsg *responseMsg, NSError *error) {
                                     if (error) {
                                         NSLog(@"sendToDevice-error:%@",error);
                                         [MBProgressHUD showError:[NSString stringWithFormat:@"%@", error.userInfo[@"errorInfo"]]];
                                         [self updateLightState:(type == LightOperationOpen ? LightOperationClose : LightOperationOpen)
                                                  withSubDomain:deMsg.device.subDomain
                                                       deviceId:deMsg.device.deviceId];
                                     }else {
                                         NSLog(@"sendToDevice - 成功");
                                         [self updateLightState:type
                                                  withSubDomain:deMsg.device.subDomain
                                                       deviceId:deMsg.device.deviceId];
                                     }
                                 }];
}

/**
 *  修改设备开关灯状态
 */
- (void)updateLightState:(LightOperationType)state
           withSubDomain:(NSString *)subDomain
                deviceId:(NSInteger)deviceId {
    //更改对应设备的开关灯状态
    for (DeviceMsg *deviceMsg in self.deviceList) {
        if ([deviceMsg.device.subDomain isEqual:subDomain] &&
            deviceMsg.device.deviceId == deviceId) {
            deviceMsg.isLightOn = state;
            [self.tableView reloadData];
        }
    }
}


#pragma UIActionSheet 代理方法

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (buttonIndex == 0) {
        //查看设备信息
        [self showTimeLine];
    } else if(buttonIndex == 1) {
        //重新设置Wi-Fi
        UIAlertView *alertView = [[UIAlertView alloc]initWithTitle:@"重新设置Wi-Fi"
                                                           message:@"请输入Wi-Fi信息"
                                                          delegate:self
                                                 cancelButtonTitle:@"取消"
                                                 otherButtonTitles:@"确定", nil];
        alertView.alertViewStyle = UIAlertViewStyleLoginAndPasswordInput;
        [[alertView textFieldAtIndex:0] setText:[self getWifiName]];
        [[alertView textFieldAtIndex:0] setPlaceholder:@"请输入Wi-Fi名称"];
        [[alertView textFieldAtIndex:1] setPlaceholder:@"请输入Wi-Fi密码"];
        [alertView show];
    } else if (buttonIndex == 2) {
        [self.navigationController pushViewController:[[TimerMsgSendingViewController alloc]
                                                       initWithDevice:self.deviceList[self.selectIndexPath.row].device]
                                             animated:YES];
    } else if (buttonIndex == 3 || buttonIndex == 4) {
        ACOTACheckInfoType type = (buttonIndex == 3 ? ACOTACheckInfoTypeSystem : ACOTACheckInfoTypeCustom);
        ACUserDevice *device = self.deviceList[self.selectIndexPath.row].device;
        ACOTACheckInfo *checkInfo = [ACOTACheckInfo checkInfoWithDeviceId:device.deviceId
                                                                  otaType:type];
        [ACOTAManager checkUpdateWithSubDomain:device.subDomain
                                  OTACheckInfo:checkInfo
                                      callback:^(ACOTAUpgradeInfo *upgradeInfo, NSError *error) {
                                          if (!upgradeInfo.update) {
                                              [MBProgressHUD showError:[NSString stringWithFormat:@"暂无新版本固件！当前版本：%@", upgradeInfo.oldVersion]];
                                              return;
                                          }
                                          [ACOTAManager confirmUpdateWithSubDomain:device.subDomain
                                                                          deviceId:device.deviceId
                                                                        newVersion:upgradeInfo.targetVersion
                                                                           otaType:type
                                                                          callback:^(NSError *error) {
                                                                              [MBProgressHUD showMessage:error ? @"更新失败！" : @"更新成功！"];
                                                                          }];
                                      }];
    }
}

- (void)showTimeLine { // 获取历史开关灯记录
    ACUserDevice *deviceMsg = self.currectDevice.device;
    @ACWeakify(self)
    //构造历史记录搜索条件
    ACDevicePropertySearchOption *option = [[ACDevicePropertySearchOption alloc]
                                            initWithSubDomain:deviceMsg.subDomain
                                            deviceId:deviceMsg.deviceId
                                            startTime:0
                                            endTime:[NSDate date].timeIntervalSince1970
                                            limit:20];
    [ACDeviceDataManager fetchHistoryPropDataWithOption:option
                                               callback:^(NSArray<ACDevicePropertyMessage *> *records,
                                                          NSError *error) {
                                                   @ACStrongify(self)
                                                   if (error) {
                                                       return;
                                                   }
                                                   NSMutableArray  *list = [NSMutableArray array];
                                                   for (ACDevicePropertyMessage *record in records) {
                                                       NSString *str = [NSString stringWithFormat:@"%@ %@",
                                                                        [self getFormatterTimer:record.timestamp],
                                                                        [record.payload getBool:@"switch"]? @"开":@"关"];
                                                       [list addObject:str];
                                                   }
                                                   [self.navigationController pushViewController:[ShowTimeLine showTimeLine:list]
                                                                                        animated:YES];
                                               }];
}

- (NSString *)getFormatterTimer:(long long)time {
    NSDate *date = [[NSDate alloc]initWithTimeIntervalSince1970:time/1000.0];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    dateFormatter.dateFormat = @"MM-dd HH:mm:ss";
    return [dateFormatter stringFromDate:date];
}

#pragma mark 获取当前Wi-Fi的名字
- (NSString *)getWifiName {
    NSString *wifiName = nil;
    CFArrayRef wifiInterfaces = CNCopySupportedInterfaces();
    if (!wifiInterfaces) {
        return nil;
    }
    NSArray *interfaces = (__bridge NSArray *)wifiInterfaces;
    for (NSString *interfaceName in interfaces) {
        CFDictionaryRef dictRef = CNCopyCurrentNetworkInfo((__bridge CFStringRef)(interfaceName));
        if (dictRef) {
            NSDictionary *networkInfo = (__bridge NSDictionary *)dictRef;
            wifiName = [networkInfo objectForKey:(__bridge NSString *)kCNNetworkInfoKeySSID];
            CFRelease(dictRef);
        }
    }
    CFRelease(wifiInterfaces);
    return wifiName;
}

#pragma mark - online state

- (NSInteger)statusWithOriginStatus:(NSInteger)originStatus localFound:(BOOL)found {
    NSInteger status = 0;
    switch (originStatus) {
        case ACDeviceStatusOffline:
            status = found ? ACDeviceStatusLocalOnline : ACDeviceStatusOffline;
            break;
        case ACDeviceStatusNetworkOnline:
            status = found ? ACDeviceStatusBothOnline : ACDeviceStatusNetworkOnline;
            break;
        case ACDeviceStatusLocalOnline:
            status = found ? ACDeviceStatusLocalOnline : ACDeviceStatusOffline;
            break;
        case ACDeviceStatusBothOnline:
            status = found ? ACDeviceStatusBothOnline : ACDeviceStatusNetworkOnline;
            break;
    }
    return status;
}

#pragma mark - lazy

- (NSMutableArray<DeviceMsg *> *)deviceList {
    @synchronized (_deviceList) {
        if (_deviceList == nil) {
            _deviceList = [NSMutableArray array];
        }
    }
    return _deviceList;
}

@end
