//
//  TimerMsgSendingViewController.m
//  ac-service-ios-Demo
//
//  Created by fariel huang on 2016/12/10.
//  Copyright © 2016年 OK. All rights reserved.
//

#import "TimerMsgSendingViewController.h"
#import "ACloudLib.h"
#import "ACUserDevice.h"
#import "ACBindManager.h"
#import "ACDeviceClient.h"

@interface TimerMsgSendingViewController() <UITableViewDelegate, UITableViewDataSource>

/** 定时发送时间输入框 */
@property (weak, nonatomic) IBOutlet UITextField *sencondsTextField;
/** 接收消息数量 */
@property (weak, nonatomic) IBOutlet UILabel *receivedLabel;
/** 发送消息数量 */
@property (weak, nonatomic) IBOutlet UILabel *sentLabel;
/** 开始按钮 */
@property (weak, nonatomic) IBOutlet UIButton *startButton;
/** msg code 输入框 */
@property (weak, nonatomic) IBOutlet UITextField *msgCodeTextField;
/** 发送数据输入框 */
@property (weak, nonatomic) IBOutlet UITextView *payloadTextView;
/** 交互历史记录列表 */
@property (weak, nonatomic) IBOutlet UITableView *historyTableView;
/** 定时发送timer */
@property (nonatomic, strong) NSTimer *sendTimer;
/** 控制的设备 */
@property (nonatomic, strong) ACUserDevice *device;
/** 已经发送的信息列表 */
@property (nonatomic, strong) NSMutableArray<ACDeviceMsg *> *sendMsgArray;
/** 响应信息列表 */
@property (nonatomic, strong) NSMutableArray *recvMsgArray;

@end

@implementation TimerMsgSendingViewController

- (instancetype)initWithDevice:(ACUserDevice *)device {
    if (self = [super init]) {
        UIStoryboard* mainStoryboard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
        self = [mainStoryboard instantiateViewControllerWithIdentifier:@"TimerMsgSendingViewController"];
        _device = device;
    }
    return self;
}

- (void)viewDidLoad {
    _historyTableView.delegate = self;
    _historyTableView.dataSource = self;
}

- (void)viewDidDisappear:(BOOL)animated {
    [_sendTimer invalidate];
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}

#pragma mark - UITableViewDelegate && UITableViewDataSource

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 80.0f;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.sendMsgArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellId=@"cell";
    UITableViewCell *cell = [_historyTableView dequeueReusableCellWithIdentifier:cellId];
    if(!cell){
        cell=[[UITableViewCell alloc]
              initWithStyle:UITableViewCellStyleSubtitle
              reuseIdentifier:cellId];
        cell.textLabel.lineBreakMode = cell.detailTextLabel.lineBreakMode = NSLineBreakByWordWrapping;
        cell.textLabel.numberOfLines = cell.detailTextLabel.numberOfLines = 0;
        cell.textLabel.font = cell.detailTextLabel.font = [UIFont systemFontOfSize:13.0f];
    }
    NSInteger index = self.sendMsgArray.count - indexPath.row -1;
    cell.textLabel.text = [NSString stringWithFormat:@"发送:%@",
                           [self.sendMsgArray[index].description
                            stringByReplacingOccurrencesOfString:@"\n"
                            withString:@""]] ;
    NSObject *resp = self.recvMsgArray[index];
    if ([resp isKindOfClass:[NSError class]]) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"响应:%@", resp] ;
    } else if ([resp isKindOfClass:[ACDeviceMsg class]]) {
        cell.detailTextLabel.text = [NSString stringWithFormat:@"响应:%@",
                                     [resp.description
                                      stringByReplacingOccurrencesOfString:@"\n"
                                      withString:@""]] ;
    } else {
        cell.detailTextLabel.text = @"未收到响应";
    }
    
    return cell;
}

#pragma mark - 定时发送数据

- (IBAction)startButtonClicked:(UIButton *)sender {
    [self.view endEditing:YES];
    [_sendTimer invalidate];
    NSTimeInterval interval = _sencondsTextField.text.length ? _sencondsTextField.text.floatValue : 5;
    _sendTimer = [NSTimer timerWithTimeInterval:interval
                                         target:self
                                       selector:@selector(sendMsgToDevice:)
                                       userInfo:nil repeats:YES];
    [[NSRunLoop currentRunLoop]addTimer:_sendTimer forMode:NSRunLoopCommonModes];
    [self sendMsgToDevice:_sendTimer];
}

- (void)sendMsgToDevice:(NSTimer *)timer {
    //构造数据
    ACDeviceMsg *msg = [[ACDeviceMsg alloc] init];
    msg.msgCode = _msgCodeTextField.text.length?_msgCodeTextField.text.integerValue:68;
    NSArray *hexArray = [_payloadTextView.text componentsSeparatedByString:@" "];
    if (hexArray.count == 0) {
        hexArray = [_payloadTextView.text componentsSeparatedByString:@","];
    }
    NSMutableData *data = [[NSMutableData alloc] init];
    for (NSString *hex in hexArray) {
        [data appendData:[self convertHexStrToData:hex]];
    }
    msg.payload = data;
    [self.sendMsgArray addObject:msg];
    [self.recvMsgArray addObject:[[NSObject alloc] init]];
    //向设备定时发送数据
    [ACBindManager sendToDeviceWithOption:ACDeviceCommunicationOptionCloudFirst
                                SubDomain:_device.subDomain
                         physicalDeviceId:_device.physicalDeviceId
                                      msg:msg callback:^(ACDeviceMsg *responseMsg, NSError *error) {
                                          NSObject *obj = [[NSObject alloc] init];
                                          if (error) {
                                              obj = error;
                                          } else {
                                              obj = responseMsg;
                                          }
                                          if (self.sendMsgArray.count > 0) {
                                              [self.recvMsgArray replaceObjectAtIndex:[self.sendMsgArray indexOfObject:msg]
                                                                           withObject:obj];
                                              NSInteger recvCount = 0;
                                              for (NSObject *resp in self.recvMsgArray) {
                                                  if ([resp isKindOfClass:[ACDeviceMsg class]]) {
                                                      recvCount ++;
                                                  }
                                              }
                                              _receivedLabel.text = [NSString stringWithFormat:@"接收:%td",
                                                                     recvCount];
                                              [_historyTableView reloadData];
                                          }
                                      }];
    _sentLabel.text = [NSString stringWithFormat:@"发送:%td",
                       self.sendMsgArray.count];
    [_historyTableView reloadData];
}

- (NSData *)convertHexStrToData:(NSString *)str {
    if (!str || [str length] == 0) {
        return nil;
    }
    NSMutableData *hexData = [[NSMutableData alloc] initWithCapacity:8];
    NSRange range;
    if ([str length] % 2 == 0) {
        range = NSMakeRange(0, 2);
    } else {
        range = NSMakeRange(0, 1);
    }
    for (NSInteger i = range.location; i < [str length]; i += 2) {
        unsigned int anInt;
        NSString *hexCharStr = [str substringWithRange:range];
        NSScanner *scanner = [[NSScanner alloc] initWithString:hexCharStr];
        
        [scanner scanHexInt:&anInt];
        NSData *entity = [[NSData alloc] initWithBytes:&anInt length:1];
        [hexData appendData:entity];
        
        range.location += range.length;
        range.length = 2;
    }
    return hexData;
}

#pragma mark - lazy

- (NSMutableArray<ACDeviceMsg *> *)sendMsgArray {
    if (_sendMsgArray == nil) {
        _sendMsgArray = [NSMutableArray array];
    }
    return _sendMsgArray;
}

- (NSMutableArray *)recvMsgArray {
    if (_recvMsgArray == nil) {
        _recvMsgArray = [NSMutableArray array];
    }
    return _recvMsgArray;
}

@end
