//
//  WifiLinkViewController.m
//  ac-service-ios-Demo
//
//  Created by OK on 15/2/10.
//  Copyright (c) 2015年 OK. All rights reserved.
//
#import <SystemConfiguration/CaptiveNetwork.h>
#import "WifiLinkViewController.h"
#import "ACWifiLinkManager.h"
#import "MBProgressHUD+MJ.h"
#import "ACLocalDevice.h"
#import "ACBindManager.h"
#import "UserDefaultUtils.h"
#import "CustomTableAlertView.h"

@interface WifiLinkViewController ()<UIAlertViewDelegate, UIActionSheetDelegate, CustomTableAlertViewDelegate>
//Wi-Fi 名称输入框
@property (weak, nonatomic) IBOutlet UITextField *wifiNameText;
//Wi-Fi 密码输入框
@property (weak, nonatomic) IBOutlet UITextField *wifiPassword;
//设备物理ID
@property (nonatomic, strong) NSString *physicsID;
@property(nonatomic, strong) NSString *bindcode;
//选择型号按钮
@property(nonatomic, strong) UIButton *modeButton;
//型号数组
@property(nonatomic, copy) NSDictionary *modes;
//配网
@property(nonatomic, strong) ACWifiLinkManager *wifiLinkManager;
//配网成功并且已经被绑定的设备
@property(nonatomic, strong) NSMutableArray<ACLocalDevice *> *boundedDevices;
//配网成功并且未被绑定的设备
@property(nonatomic, strong) NSMutableArray<ACLocalDevice *> *unBindDevices;

@end

@implementation WifiLinkViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.wifiNameText setText:[self getWifiName]];
    //右上角选择型号按钮
    _modeButton = [UIButton new];
    _modeButton.frame = CGRectMake(0, 0, 80, 30);
    [_modeButton setTitleColor:[UIColor blueColor] forState:UIControlStateNormal];
    [_modeButton setTitle:[UserDefaultUtils getWifiMode] forState:UIControlStateNormal];
    [_modeButton addTarget:self
                    action:@selector(chooseModeClicked:)
          forControlEvents:UIControlEventTouchUpInside];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:_modeButton];
}

- (IBAction)linkButClick:(UIButton *)sender {
    [MBProgressHUD showMessage:@"正在设置Wi-Fi..."];
    [_wifiLinkManager stopWifiLink];
    _unBindDevices = _boundedDevices = nil;
    //这里配网时可直接写死为对应的模块型号如：initWithLinkerName:ACLinkerNameHF
    _wifiLinkManager = [[ACWifiLinkManager alloc]
                        initWithLinkerName:self.modes[[UserDefaultUtils getWifiMode]]];
    NSString *ssid = self.wifiNameText.text;
    NSString *password = self.wifiPassword.text;
    [_wifiLinkManager sendWifiInfo:ssid
                          password:password
                           timeout:60.0
                          callback:^(NSArray *localDevices, NSError *error) {
                              //配网成功一台立即停止配网 若需配多台则注释此行代码
                              [_wifiLinkManager stopWifiLink];
                              if (error) {
                                  NSLog(@"sendWifiInfo-error: %@",error);
                                  [MBProgressHUD hideHUD];
                                  [MBProgressHUD showError:@"设置失败"];
                              } else {
                                  [MBProgressHUD hideHUD];
                                  if (localDevices.count>0) {
                                      ACLocalDevice *localdevice = [localDevices lastObject];
                                      NSLog(@"配网发现设备:%@", localdevice);
                                      [self updateWifiDeviceListWithDevice:localdevice];
                                  } else {
                                      [MBProgressHUD showError:@"没有可发现设备"];
                                  }
                              }
                          }];
}

/**
 更新发现设备列表
 */
- (void)updateWifiDeviceListWithDevice:(ACLocalDevice *)localdevice {
    //判断设备是否已经被绑定过
    [ACBindManager isDeviceBoundsWithSubDomain:[UserDefaultUtils getSubDomain]
                              physicalDeviceId:localdevice.deviceId
                                      callback:^(BOOL isBounded, NSError *error) {
                                          if (error) {
                                              NSLog(@"%@", error.userInfo);
                                              [MBProgressHUD showError:[NSString stringWithFormat:@"设备绑定状态获取失败：%@",
                                                                        error.userInfo]];
                                              return;
                                          }
                                          if (isBounded) {
                                              if (![self.boundedDevices containsObject:localdevice]) {
                                                  [self.boundedDevices addObject:localdevice];
                                              }
                                          } else {
                                              if (![self.unBindDevices containsObject:localdevice]) {
                                                  [self.unBindDevices addObject:localdevice];
                                              }
                                          }
                                          //拼接所有发现设备的物理id 并标记是否已被绑定过
                                          NSMutableArray *deviceNames = @[].mutableCopy;
                                          for (ACLocalDevice *device in self.unBindDevices) {
                                              [deviceNames addObject:device.deviceId];
                                          }
                                          for (ACLocalDevice *device in self.boundedDevices) {
                                              [deviceNames addObject:[NSString stringWithFormat:@"%@(已被绑定)", device.deviceId]];
                                          }
                                          //弹出选择绑定框
                                          if (![CustomTableAlertView isDisplaying]) {
                                              [CustomTableAlertView alertWithTarget:self
                                                                         title:@"配网成功的设备"
                                                                       options:deviceNames
                                                                       buttons:@[@"绑定设备"]
                                                                      callback:^BOOL(NSInteger selectedIndex,
                                                                                     NSInteger buttonIndex) {
                                                                          if (buttonIndex < 0) {
                                                                              [MBProgressHUD showError:@"请选择要绑定的设备!"];
                                                                              return NO;
                                                                          }
                                                                          //绑定选中设备
                                                                          [self bindDevice:self.unBindDevices[selectedIndex]];
                                                                          //取消配网
                                                                          [_wifiLinkManager stopWifiLink];
                                                                          return YES;
                                                                      }];
                                          } else {
                                              [CustomTableAlertView updateOptions:deviceNames];
                                          }
                                      }];
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
    [MBProgressHUD hideHUD];
    [_wifiLinkManager stopWifiLink];
}

/**
 *  选择型号
 */
- (void)chooseModeClicked:(UIButton *)sender {
    UIActionSheet *sheet = [UIActionSheet new];
    sheet.delegate = (id)self;
    sheet.title = @"提示";
    [self.modesSortedKeys enumerateObjectsUsingBlock:^(id  _Nonnull keyName,
                                                     NSUInteger idx,
                                                     BOOL * _Nonnull stop) {
        [sheet addButtonWithTitle:keyName];
    }];
    [sheet addButtonWithTitle:@"取消"];
    [sheet setCancelButtonIndex:self.modes.count];
    [sheet showInView:self.view];
}

/**
 *  连接设备
 */
-(void)bindDevice:(ACLocalDevice *)device {
    NSString *subDomain = [UserDefaultUtils getSubDomain];
    [ACBindManager bindDeviceWithSubDomain:subDomain
                          physicalDeviceId:device.deviceId
                                      name:self.name.text
                                  callback:^(ACUserDevice *userDevice, NSError *error) {
        if (error) {
            NSLog(@"bindDevice－－－error: %@", error);
            [MBProgressHUD showError:@"绑定失败"];
        } else {
            NSLog(@"bindDevice－－ACUserDevice:%@",userDevice);
            [MBProgressHUD showSuccess:@"绑定成功"];
        }
    }];
}

#pragma mark - CustomAlertViewDelegate

- (void)customTableAlertViewCell:(UITableViewCell *)cell atIndex:(NSInteger)index {
    if (index > ((int)self.unBindDevices.count - 1)) {
        //已被绑定的设备为不可选中状态
        cell.userInteractionEnabled = NO;
    } else {
        cell.userInteractionEnabled = YES;
    }
}

#pragma mark 获取当前Wi-Fi的名字
- (NSString *)getWifiName {
    NSString *wifiName = nil;
    CFArrayRef wifiInterfaces = CNCopySupportedInterfaces();
    if (!wifiInterfaces) {
        return nil;
    }
    NSArray *interfaces = (__bridge NSArray *)wifiInterfaces;
    for (NSString *interfaceName in interfaces) {
        CFDictionaryRef dictRef = CNCopyCurrentNetworkInfo((__bridge CFStringRef)(interfaceName));
        if (dictRef) {
            NSDictionary *networkInfo = (__bridge NSDictionary *)dictRef;
            wifiName = [networkInfo objectForKey:(__bridge NSString *)kCNNetworkInfoKeySSID];
            CFRelease(dictRef);
        }
    }
    CFRelease(wifiInterfaces);
    return wifiName;
}

#pragma mark - UIActionSheetDelegate

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
    if (buttonIndex < self.modes.count) {
        [UserDefaultUtils setWifiMode:self.modesSortedKeys[buttonIndex]];
        [_modeButton setTitle:[UserDefaultUtils getWifiMode] forState:UIControlStateNormal];
    }
}

#pragma mark - lazy 

- (NSDictionary *)modes {
    if (_modes == nil) {
        _modes = @{
                   @"汉枫" : ACLinkerNameHF,
                   @"MTK" : ACLinkerNameMTK,
                   @"MARVELL" : ACLinkerNameMarvell,
                   @"RAK" : ACLinkerNameRAK,
                   @"TI" : ACLinkerNameTI,
                   @"乐鑫" : ACLinkerNameESPTouch,
                   @"瑞昱" : ACLinkerNameRealtek,
                   @"庆科" : ACLinkerNameEasyLink,
                   @"联胜德" : ACLinkerNameOneshot,
                   @"古北" : ACLinkerNameGuBei,
                   @"新力维" : ACLinkerNameXLW,
                   @"AP模式" : ACLinkerNameAPMode,
                   @"Android" : ACLinkerNameAndroid
                   };
    }
    return _modes;
}

- (NSMutableArray<ACLocalDevice *> *)boundedDevices {
    if (_boundedDevices == nil) {
        _boundedDevices = @[].mutableCopy;
    }
    return _boundedDevices;
}

- (NSMutableArray<ACLocalDevice *> *)unBindDevices {
    if (_unBindDevices == nil) {
        _unBindDevices = @[].mutableCopy;
    }
    return _unBindDevices;
}

- (NSArray *)modesSortedKeys {
    return [self.modes.allKeys sortedArrayUsingComparator:^NSComparisonResult(id  _Nonnull obj1,
                                                                              id  _Nonnull obj2) {
        return [obj1 compare:obj2 options:NSNumericSearch];
    }];
}

@end
